@@ -355,7 +355,11 @@ class MozillaDelegate(userprofile.SourceDelegate):
                                          {"file_type" : FirefoxProfileFile.TYPE_BOOKMARK,
                                           "mandatory"    : True})
 
-    def apply(self):
+    def set_enforce_mandatory (self, enforce):
+        # Nothing to do here
+        pass
+
+    def apply(self, is_sabayon_session):
         ini_files = []
         pref_files = []
         bookmark_files = []
@@ -402,7 +406,7 @@ class MozillaDelegate(userprofile.SourceDelegate):
         if sabayon_pref_rel_path in pref_files:
             dprint(LOG_APPLY, "extracting %s" % sabayon_pref_rel_path)
             self.source.storage.extract(sabayon_pref_rel_path, self.home_dir, True)
-            apply_pref = JavascriptPrefsFile(self.home_dir, path)
+            apply_pref = JavascriptPrefsFile(self.home_dir, sabayon_pref_rel_path)
             apply_pref.read()
         else:
             apply_pref = None
@@ -410,7 +414,7 @@ class MozillaDelegate(userprofile.SourceDelegate):
         if sabayon_mandatory_pref_rel_path in pref_files:
             dprint(LOG_APPLY, "extracting %s" % sabayon_mandatory_pref_rel_path)
             self.source.storage.extract(sabayon_mandatory_pref_rel_path, self.home_dir, True)
-            mandatory_apply_pref = JavascriptPrefsFile(self.home_dir, path)
+            mandatory_apply_pref = JavascriptPrefsFile(self.home_dir, sabayon_mandatory_pref_rel_path)
             mandatory_apply_pref.read()
         else:
             mandatory_apply_pref = None
@@ -420,7 +424,7 @@ class MozillaDelegate(userprofile.SourceDelegate):
         if sabayon_bookmark_rel_path in bookmark_files:
             dprint(LOG_APPLY, "extracting %s" % sabayon_bookmark_rel_path)
             self.source.storage.extract(sabayon_bookmark_rel_path, self.home_dir, True)
-            apply_bookmark = BookmarksFile(self.home_dir, path)
+            apply_bookmark = BookmarksFile(self.home_dir, sabayon_bookmark_rel_path)
             apply_bookmark.read()
         else:
             apply_bookmark = None
@@ -428,7 +432,7 @@ class MozillaDelegate(userprofile.SourceDelegate):
         if sabayon_mandatory_bookmark_rel_path in bookmark_files:
             dprint(LOG_APPLY, "extracting %s" % sabayon_mandatory_bookmark_rel_path)
             self.source.storage.extract(sabayon_mandatory_bookmark_rel_path, self.home_dir, True)
-            mandatory_apply_bookmark = BookmarksFile(self.home_dir, path)
+            mandatory_apply_bookmark = BookmarksFile(self.home_dir, sabayon_mandatory_bookmark_rel_path)
             mandatory_apply_bookmark.read()
         else:
             mandatory_apply_bookmark = None
@@ -633,13 +637,15 @@ class JavascriptPrefsFile(FirefoxProfileFile):
                                                            src_pref.get_value())
                 
 
-    def read(self):
+    def read(self, full_path = None):
         self.prev_prefs = self.get_prefs()
 
+        if not full_path:
+            full_path = self.get_full_path()
         dprint(LOG_OPERATION, "read profile prefs (%s)", self.get_full_path())
         self.file_state = file_state.UNKNOWN
         try:
-            fd = open(self.get_full_path())
+            fd = open(full_path)
         except IOError, e:
             if e.errno == errno.ENOENT:
                 self.file_state = file_state.NOT_FOUND
@@ -677,14 +683,24 @@ class JavascriptPrefsFile(FirefoxProfileFile):
         emit_changes(_del.items(), MozillaChange.DELETED)
         emit_changes(_mod.items(), MozillaChange.CHANGED)
 
-    def kill_comments(self):
-        slash_comment_re = re.compile("//.*$", re.MULTILINE)
-        hash_comment_re = re.compile("#.*$", re.MULTILINE)
-        c_comment_re = re.compile("/\*.*?\*/", re.MULTILINE | re.DOTALL)
+        
 
-        self.filebuf = slash_comment_re.sub("", self.filebuf)
-        self.filebuf = hash_comment_re.sub("", self.filebuf)
-        self.filebuf = c_comment_re.sub("", self.filebuf)
+    def kill_comments(self):
+        def not_in_string (regexp):
+            double = '\\"(?:\\\\.|[^\"\\\\])*\\"'
+            single = "\\'(?:\\\\.|[^\"\\\\])*\\'"
+            return "(" + double + "|" + single + ")(?:" + regexp + ")"
+        def match (matchobj):
+            return matchobj.group(1)
+        
+            
+        slash_comment_re = re.compile(not_in_string ("//.*$"), re.MULTILINE)
+        hash_comment_re = re.compile(not_in_string ("#.*$"), re.MULTILINE)
+        c_comment_re = re.compile(not_in_string ("/\*.*?\*/"), re.MULTILINE | re.DOTALL)
+
+        self.filebuf = slash_comment_re.sub(match, self.filebuf)
+        self.filebuf = hash_comment_re.sub(match, self.filebuf)
+        self.filebuf = c_comment_re.sub(match, self.filebuf)
 
     def parse(self):
         start = 0;
@@ -1120,6 +1136,3 @@ def run_unit_tests():
     test_prefs = {'foo':'"bar"', 'uno':'1'}
 
     dprint(LOG_OPERATION, "In mozillaprofile tests")
-
-
-
