@@ -18,6 +18,7 @@
 # Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #
 
+import os
 import gconf
 import gconfsource
 
@@ -33,12 +34,16 @@ except:
 def dprint (fmt, *args):
     util.debug_print (util.DEBUG_PANELDELEGATE, fmt % args)
 
+PANEL_LAUNCHER_DIR = ".gnome2/panel2.d/default/launchers"
+
 class PanelChange (userprofile.ProfileChange):
     def __init__ (self, source, delegate, id):
         userprofile.ProfileChange.__init__ (self, source, delegate)
         self.id = id
     def get_id (self):
         return self.id
+    def commit_change (self, mandatory):
+        pass
 
 #
 # FIXME: these short descriptions are lame, should be:
@@ -75,12 +80,23 @@ class PanelObjectAddedChange (PanelChange):
         PanelChange.__init__ (self, source, delegate, id)
     def get_short_description (self):
         return _("Panel object '%s' added") % self.id
+    def commit_change (self, mandatory):
+        # Might have to commit a launcher file
+        launcher = self.delegate.client.get_string (PANEL_KEY_BASE + "/objects/" + self.id + "/launcher_location")
+        if launcher and launcher[0] != '/':
+            file = PANEL_LAUNCHER_DIR + "/" + launcher
+            self.source.storage.add (file, self.delegate.home_dir, self.delegate.name)
     
 class PanelObjectRemovedChange (PanelChange):
     def __init__ (self, source, delegate, id):
         PanelChange.__init__ (self, source, delegate, id)
     def get_short_description (self):
         return _("Panel object '%s' removed") % self.id
+    def commit_change (self, mandatory):
+        launcher = self.delegate.client.get_string (PANEL_KEY_BASE + "/objects/" + self.id + "/launcher_location")
+        if launcher and launcher[0] != '/':
+            file = PANEL_LAUNCHER_DIR + "/" + launcher
+            self.source.storage.remove (file)
 
 class PanelDelegate (userprofile.SourceDelegate):
     class PanelThing:
@@ -97,7 +113,7 @@ class PanelDelegate (userprofile.SourceDelegate):
             # self.name        = self.client.get_string (PANEL_KEY_BASE + "/toplevels/" + toplevel_id + "/name")
             # self.orientation = self.client.get_string (PANEL_KEY_BASE + "/toplevels/" + toplevel_id + "/orientation")
             # self.expand      = self.client.get_bool   (PANEL_KEY_BASE + "/toplevels/" + toplevel_id + "/expand")
-        
+
     class PanelApplet (PanelThing):
         def __init__ (self, id, added = False, removed = False):
             PanelDelegate.PanelThing.__init__ (self, id, added, removed)
@@ -118,10 +134,10 @@ class PanelDelegate (userprofile.SourceDelegate):
         userprofile.SourceDelegate.__init__ (self, _("Panel"), source, PANEL_KEY_BASE)
         self.client = gconf.client_get_default ()
 
+        self.home_dir = util.get_home_dir()
         self.toplevels = {}
         self.applets = {}
         self.objects = {}
-        self.__read_panel_config ()
 
     def __read_panel_config (self):
         dprint ("Reading initial panel config");
@@ -253,6 +269,8 @@ class PanelDelegate (userprofile.SourceDelegate):
         if not thing.added:
             return
 
+        change.commit_change (mandatory)
+
         (client, address) = self.source.get_committing_client_and_address (mandatory)
 
         self.__copy_dir (self.client, client, address, PANEL_KEY_BASE + "/" + dir_name + "/" + thing.id)
@@ -271,6 +289,8 @@ class PanelDelegate (userprofile.SourceDelegate):
         if not thing.removed:
             return
 
+        change.commit_change (mandatory)
+
         (client, address) = self.source.get_committing_client_and_address (mandatory)
 
         id_list = self.__get_current_list (dict)
@@ -319,9 +339,12 @@ class PanelDelegate (userprofile.SourceDelegate):
                                           "object_id_list",
                                           "objects")
 
+    def get_path_description (self, path):
+        return "Panel launcher: %s"%os.path.basename(path)
+    
     def start_monitoring (self):
-        """Start monitoring for configuration changes."""
-        # Nothing to do here
+        # Need to do this here so gconf paths are setup
+        self.__read_panel_config ()
         pass
 
     def stop_monitoring (self):
@@ -333,13 +356,47 @@ class PanelDelegate (userprofile.SourceDelegate):
         # Nothing to do here
         pass
     
-    def apply (self):
+    def set_enforce_mandatory (self, enforce):
         # Nothing to do here
         pass
 
+    def __apply_foreach (self, source, path):
+        self.source.storage.extract (path, self.home_dir)
+
+    def apply (self, is_sabayon_session):
+        self.source.storage.foreach (self.__apply_foreach, source = self.name)
+        pass
+
 def get_gconf_delegate (source):
     return PanelDelegate (source)
 
+class PanelFileDelegate (userprofile.SourceDelegate):
+    def __init__ (self, source):
+        userprofile.SourceDelegate.__init__ (self, _("Panel File"), source, PANEL_LAUNCHER_DIR)
+
+    def handle_change (self, change):
+        dprint ("Ignoring file chage due to panel delegation: %s"%change)
+        return True
+
+    def start_monitoring (self):
+        pass
+
+    def stop_monitoring (self):
+        pass
+
+    def sync_changes (self):
+        pass
+
+    def set_enforce_mandatory (self, enforce):
+        pass
+
+    def apply (self, is_sabayon_session):
+        pass
+
+def get_files_delegate(source):
+    return PanelFileDelegate(source)
+
+
 #
 # Unit tests
 #
